@@ -3,9 +3,9 @@ use warnings;
 use v5.14;
 
 package DBIx::Poggy;
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
-use Scalar::Util qw(weaken);
+use Scalar::Util qw(weaken refaddr);
 
 =head1 NAME
 
@@ -105,6 +105,7 @@ sub new {
 sub init {
     my $self = shift;
     $self->{pool_size} ||= 10;
+    $self->{ping_on_take} ||= 30;
     return $self;
 }
 
@@ -119,6 +120,7 @@ sub connect {
     my ($dsn, $user, $password, $opts) = @_;
 
     $opts ||= {};
+    $opts->{RaiseError} //= 1;
 
     $self->{free} ||= [];
 
@@ -135,6 +137,7 @@ sub _connect {
         @{ $self->{connection_settings} }
     ) or die DBIx::Poggy::Error->new( 'DBIx::Poggy::DBI' );
     push @{$self->{free}}, $dbh;
+    $self->{last_used}{ refaddr $dbh } = time;
 
     return;
 }
@@ -167,12 +170,27 @@ sub take {
             errstr => 'Attempt to take a connection from not initialized pool',
         );
     }
-    unless ( @{ $self->{free} } ) {
-        warn "DB pool exhausted, creating a new connection";
-        $self->_connect;
+    my $dbh;
+    while (1) {
+        unless ( @{ $self->{free} } ) {
+            warn "DB pool exhausted, creating a new connection";
+            $self->_connect;
+            $dbh = shift @{ $self->{free} };
+            delete $self->{last_used}{ refaddr $dbh };
+            last;
+        }
+
+        $dbh = shift @{ $self->{free} };
+        my $used = delete $self->{last_used}{ refaddr $dbh };
+        if ( (time - $used) > $self->{ping_on_take} ) {
+            unless ( $dbh->ping ) {
+                warn "connection is not alive, dropping";
+                next;
+            }
+        }
+        last;
     }
 
-    my $dbh = shift @{ $self->{free} };
     if ( $args{auto} ) {
         $dbh->{private_poggy_state}{release_to} = $self;
         weaken $dbh->{private_poggy_state}{release_to};
@@ -191,7 +209,16 @@ no protection against double putting or active queries on the handle.
 
 sub release {
     my $self = shift;
-    push @{ $self->{free} }, shift;
+    my $dbh = shift;
+    delete $dbh->{private_poggy_state}{release_to};
+
+    if ( $dbh->err && !$dbh->ping ) {
+        warn "handle is in error state and not ping'able, not releasing to the pool";
+        return $self;
+    }
+
+    push @{ $self->{free} }, $dbh;
+    $self->{last_used}{ refaddr $dbh } = time;
     return $self;
 }
 
